//
//   File : kvi_ircserverdb.cpp
//   Creation date : Mon Jul 10 2000 14:25:00 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#define __LIBKVILIB__

#include <qapplication.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qcheckbox.h>

#include "kvi_ircserverdb.h"
#include "kvi_config.h"
#include "kvi_locale.h"
#include "kvi_netutils.h"






// NETWORK NAMES , SERVER NAMES & SERVER DESCRIPTIONS are in Latin1 charset

KviIrcServerDataBase::KviIrcServerDataBase()
{
	m_pNetworkList = new KviPtrList<KviIrcNetwork>;
	m_pNetworkList->setAutoDelete(true);
	m_pCurrentNetwork = 0;
}

KviIrcServerDataBase::~KviIrcServerDataBase()
{
	delete m_pNetworkList;
}

void KviIrcServerDataBase::clear()
{
//	delete m_pNetworkList;
//	m_pNetworkList = new KviPtrList<KviIrcNetwork>;
//	m_pNetworkList->setAutoDelete(true);
	m_pNetworkList->clear();
	m_pCurrentNetwork = 0;
}

void KviIrcServerDataBase::insertNetwork(KviIrcNetwork *n)
{
	int i = 0;
	for(KviIrcNetwork * net = m_pNetworkList->first();net;net = m_pNetworkList->next())
	{
		if(kvi_strcmpCI(net->m_szName.ptr(),n->m_szName.ptr()) > 0)
		{
			m_pNetworkList->insert(i,n);
			return;
		}
		i++;
	}
	m_pNetworkList->append(n);
}

KviIrcNetwork * KviIrcServerDataBase::findNetwork(const char * name)
{
	for(KviIrcNetwork * net = m_pNetworkList->first();net;net = m_pNetworkList->next())
	{
		if(kvi_strEqualCI(net->m_szName.ptr(),name))return net;
	}
	return 0;
}

void KviIrcServerDataBase::updateServerIp(const char * server,unsigned short int port,const char *ip)
{
	for(KviIrcNetwork * net = m_pNetworkList->first();net;net = m_pNetworkList->next())
	{
		KviIrcServer * srv = net->findServer(server,port);
		if(srv)
		{
			srv->m_szIp = ip;
			return;
		}
	}
}

//void KviIrcServerDataBase::disableCaching(const char * server)
//{
//	for(KviIrcNetwork * net = m_pNetworkList->first();net;net = m_pNetworkList->next())
//	{
//		KviIrcServer * srv = net->findServer(server);
//		if(srv)
//		{
//			srv->setCacheIp(false);
//		}
//	}
//}

//void KviIrcServerDataBase::iWantThisServerToBeTheCurrent(const char * server,unsigned short int uPort,bool bPortIsValid,bool bIpV6,bool bSSL,const char * linkFilter)
void KviIrcServerDataBase::makeCurrentServer(KviIrcServerDefinition * d)
{
	for(KviIrcNetwork * net = m_pNetworkList->first();net;net = m_pNetworkList->next())
	{
		KviIrcServer * srv = net->findServer(d->szServer.ptr(),d->uPort);
		if(srv)
		{
			if((d->bIpV6 == srv->isIpV6()) && (d->bSSL == srv->useSSL()))
			{
				bool bThisOne = true;
				if(d->szLinkFilter.hasData())
				{
					if(!kvi_strEqualCI(d->szLinkFilter.ptr(),srv->m_szLinkFilter.ptr()))bThisOne = false;
				}
				if(bThisOne)
				{
					if(d->bPortIsValid)srv->setPort(d->uPort);
					if(d->szNick.hasData())srv->m_szNick = d->szNick;
					if(d->szPass.hasData())srv->m_szPass = d->szPass; // don't clear the pass!
					if(d->szInitUMode.hasData())srv->m_szInitUMode = d->szInitUMode;
					setCurrentNetwork(net);
					net->setCurrentServer(srv);
					return;
				}
			}
		}
	}

	// no such server: is it a valid ip address or hostname ?
	bool bIsValidIpV4 = kvi_isValidStringIp(d->szServer.ptr());
#ifdef COMPILE_IPV6_SUPPORT
	bool bIsValidIpV6 = kvi_isValidStringIp_V6(d->szServer.ptr());
#else
	bool bIsValidIpV6 = false;
#endif

	if(!(bIsValidIpV4 || bIsValidIpV6))
	{
		// is it a valid hostname ? (must contain at least one dot)
		if(!d->szServer.contains('.'))
		{
			// assume it is a network name!
			KviIrcNetwork * n = findNetwork(d->szServer.ptr());
			if(n)
			{
				setCurrentNetwork(n);

				// find a round-robin server in that network

				for(KviIrcServer * s = n->m_pServerList->first();s;s = n->m_pServerList->next())
				{
					if(s->m_szDescription.contains("random",false) ||
						(s->m_szDescription.contains("round",false) && s->m_szDescription.contains("robin",false)))
					{
						n->setCurrentServer(s);
						return;
					}
				}

				// no explicit round robin... try some common names

				KviStr tryAlso1(KviStr::Format,"irc.%s.org",d->szServer.ptr());
				KviStr tryAlso2(KviStr::Format,"irc.%s.net",d->szServer.ptr());
				KviStr tryAlso3(KviStr::Format,"irc.%s.com",d->szServer.ptr());

				for(KviIrcServer * s = n->m_pServerList->first();s;s = n->m_pServerList->next())
				{
					if(kvi_strEqualCI(s->m_szHostname.ptr(),tryAlso1.ptr()) ||
						kvi_strEqualCI(s->m_szHostname.ptr(),tryAlso2.ptr()) ||
						kvi_strEqualCI(s->m_szHostname.ptr(),tryAlso3.ptr()))
					{
						n->setCurrentServer(s);
						return;
					}
				}

				// a random one in this network
				return;
			} // else unreasonable..but :D
		}
	}

	// a valid hostname or ip address , not found in list : add it and make it current

	KviIrcNetwork * n = findNetwork(__tr("orphan_servers"));
	if(!n)
	{
		n = new KviIrcNetwork(__tr("orphan_servers"));
		insertNetwork(n);
	}
	KviIrcServer * s = new KviIrcServer();
	s->m_szHostname = d->szServer;
	if(bIsValidIpV4)
	{
		s->m_szIp = d->szServer;
		s->setCacheIp(true);
#ifdef COMPILE_IPV6_SUPPORT
	} else {
		if(bIsValidIpV6)
		{
			s->m_szIp = d->szServer;
			s->setCacheIp(true);
			d->bIpV6 = true;
		}
	}
#else
	}
#endif
	s->m_uPort = d->bPortIsValid ? d->uPort : 6667;
	s->setLinkFilter(d->szLinkFilter.ptr());
	s->m_szPass= d->szPass;
	s->m_szNick= d->szNick;
    s->m_szInitUMode = d->szInitUMode;
	s->setIpV6(d->bIpV6);
	s->setUseSSL(d->bSSL);
	n->insertServer(s);
	setCurrentNetwork(n);
	n->setCurrentServer(s);
}

void KviIrcServerDataBase::setCurrentNetwork(KviIrcNetwork *n)
{
	if(m_pNetworkList->findRef(n) != -1)m_pCurrentNetwork = n;
}


void KviIrcServerDataBase::load(const char * filename)
{
	clear();
	KviConfig cfg(filename,KviConfig::Read);

	KviConfigIterator it(*(cfg.dict()));
	while(it.current())
	{
		if(it.current()->count() > 0)
		{
			KviIrcNetwork * n = new KviIrcNetwork(it.currentKey());
			insertNetwork(n);
			cfg.setGroup(it.currentKey());
			int nServers = cfg.readIntEntry("NServers",0);
			if(cfg.readBoolEntry("Current",false))m_pCurrentNetwork = n;
			for(int i=0;i < nServers;i++)
			{
				KviIrcServer *s = new KviIrcServer();
				n->m_pServerList->append(s);
				KviStr tmp(KviStr::Format,"%d_Current",i);
				if(cfg.readBoolEntry(tmp.ptr(),false))n->m_pCurrentServer = s;
				tmp.sprintf("%d_Hostname",i);
				s->m_szHostname = cfg.readEntry(tmp.ptr(),"irc.unknown.net");
				tmp.sprintf("%d_Ip",i);
				s->m_szIp = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_Description",i);
				s->m_szDescription = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_User",i);
				s->m_szUser = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_Pass",i);
				s->m_szPass = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_Nick",i);
				s->m_szNick = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_RealName",i);
				s->m_szRealName = cfg.readEntry(tmp.ptr(),"");
                tmp.sprintf("%d_InitUmode",i);
                s->m_szInitUMode = cfg.readEntry(tmp.ptr(),"0");

                // load channel list
                QString channels;
                tmp.sprintf("%d_LoginChannels",i);
                channels = cfg.readEntry(tmp.ptr(),"");
                if (!channels.isEmpty()) {
                    int idx=0;
                    int idx2=0;
                    while (idx2 > -1) {
                        idx2 =channels.find(';',idx);
                        if (idx != channels.length()-1) {
                            if (!(channels.mid(idx,idx2-idx)).isEmpty()) {
                                s->m_pChannelList->append (QString("#"+channels.mid(idx,idx2-idx))); // #blabla
                            }
                        }
                        idx=idx2+1;
                    }
                }

                // auto connect
                tmp.sprintf("%d_UseAutoConnect",i);
				bool bTmp = cfg.readBoolEntry(tmp.ptr(),false);
				s->setUseAutoConnect (bTmp);

                tmp.sprintf("%d_OnConnectCommand",i);
				s->m_szOnConnectCommand = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_OnLoginCommand",i);
				s->m_szOnLoginCommand = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_LinkFilter",i);
				s->m_szLinkFilter = cfg.readEntry(tmp.ptr(),"");
				tmp.sprintf("%d_Port",i);
				s->m_uPort = cfg.readUShortEntry(tmp.ptr(),6667);
				tmp.sprintf("%d_IpV6",i);
				bTmp = cfg.readBoolEntry(tmp.ptr(),false);
				s->setIpV6(bTmp);
				tmp.sprintf("%d_CacheIp",i);
				bTmp = cfg.readBoolEntry(tmp.ptr(),true);
				s->setCacheIp(bTmp);
				tmp.sprintf("%d_SSL",i);
				bTmp = cfg.readBoolEntry(tmp.ptr(),false);
				s->setUseSSL(bTmp);
			}
			if(!n->m_pCurrentServer)n->m_pCurrentServer = n->m_pServerList->first();
		}
		++it;
	}
	if(!m_pCurrentNetwork)m_pCurrentNetwork = m_pNetworkList->first();
}

void KviIrcServerDataBase::save(const char * filename)
{
	KviConfig cfg(filename,KviConfig::Write);

	cfg.clear(); // clear any old entry

	for(KviIrcNetwork * n = m_pNetworkList->first();n;n = m_pNetworkList->next())
	{
		cfg.setGroup(n->m_szName.ptr());
		cfg.writeEntry("NServers",n->m_pServerList->count());
		if(n == m_pCurrentNetwork)cfg.writeEntry("Current",true);

		int i=0;
		for(KviIrcServer *s = n->m_pServerList->first();s;s = n->m_pServerList->next())
		{
			KviStr tmp(KviStr::Format,"%d_Hostname",i);
			cfg.writeEntry(tmp.ptr(),s->m_szHostname.ptr());
			if(s->m_szIp.hasData())
			{
				tmp.sprintf("%d_Ip",i);
				cfg.writeEntry(tmp.ptr(),s->m_szIp.ptr());
			}
			if(s->m_szDescription.hasData())
			{
				tmp.sprintf("%d_Description",i);
				cfg.writeEntry(tmp.ptr(),s->m_szDescription.ptr());
			}
			if(s->m_szUser.hasData())
			{
				tmp.sprintf("%d_User",i);
				cfg.writeEntry(tmp.ptr(),s->m_szUser.ptr());
			}
			if(s->m_szPass.hasData())
			{
				tmp.sprintf("%d_Pass",i);
				cfg.writeEntry(tmp.ptr(),s->m_szPass.ptr());
			}
			if(s->m_szNick.hasData())
			{
				tmp.sprintf("%d_Nick",i);
				cfg.writeEntry(tmp.ptr(),s->m_szNick.ptr());
			}
			if(s->m_szRealName.hasData())
			{
				tmp.sprintf("%d_RealName",i);
				cfg.writeEntry(tmp.ptr(),s->m_szRealName.ptr());
			}
            //'0' is not configured, "" is just flags don't set
            if(!s->m_szInitUMode.contains('0'))
            {
                tmp.sprintf("%d_InitUMode",i);
                cfg.writeEntry(tmp.ptr(),s->m_szInitUMode.ptr());
            }

            // save channel list
            QString channels="";
            for(QStringList::Iterator it = s->m_pChannelList->begin(); it != s->m_pChannelList->end(); ++it){
                QString tmp =*it;
                QString tmp2;
                tmp2= tmp.right(tmp.length()-1); // strip #
                channels+=tmp2+";";
            }
            tmp.sprintf("%d_LoginChannels",i);
				cfg.writeEntry(tmp.ptr(),channels);

            if(s->useAutoConnect() )
			{
				tmp.sprintf("%d_UseAutoConnect",i);
				cfg.writeEntry(tmp.ptr(),s->useAutoConnect());
			}

			if(s->m_szOnConnectCommand.hasData())
			{
				tmp.sprintf("%d_OnConnectCommand",i);
				cfg.writeEntry(tmp.ptr(),s->m_szOnConnectCommand.ptr());
			}
			if(s->m_szOnLoginCommand.hasData())
			{
				tmp.sprintf("%d_OnLoginCommand",i);
				cfg.writeEntry(tmp.ptr(),s->m_szOnLoginCommand.ptr());
			}
			if(s->m_szLinkFilter.hasData())
			{
				tmp.sprintf("%d_LinkFilter",i);
				cfg.writeEntry(tmp.ptr(),s->m_szLinkFilter.ptr());
			}
			if(s->m_uPort != 6667)
			{
				tmp.sprintf("%d_Port",i);
				cfg.writeEntry(tmp.ptr(),s->m_uPort);
			}
			if(s->isIpV6())
			{
				tmp.sprintf("%d_IpV6",i);
				cfg.writeEntry(tmp.ptr(),s->isIpV6());
			}
			if(!(s->cacheIp()))
			{
				tmp.sprintf("%d_CacheIp",i);
				cfg.writeEntry(tmp.ptr(),s->cacheIp());
			}
			if(s->useSSL())
			{
				tmp.sprintf("%d_SSL",i);
				cfg.writeEntry(tmp.ptr(),s->useSSL());
			}
			if(s == n->m_pCurrentServer)
			{
				tmp.sprintf("%d_Current",i);
				cfg.writeEntry(tmp.ptr(),true);
			}
			i++;
		}
	}
}

