#ifndef _KVI_KVS_MODULEINTERFACE_H_
#define _KVI_KVS_MODULEINTERFACE_H_
//=============================================================================
//
//   File : kvi_kvs_moduleinterface.h
//   Created on Tue 16 Dec 2003 00:27:54 by Szymon Stefanek
//
//   This file is part of the KVIrc IRC client distribution
//   Copyright (C) 2003 Szymon Stefanek <pragma at kvirc dot net>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================


#include "kvi_settings.h"

#include <qdict.h>

#include "kvi_kvs_runtimecontext.h"

typedef bool (*moduleSimpleCommandExecRoutine)(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsSwitchList * pSwitches);
typedef struct _KviKvsModuleSimpleCommandExecRoutine
{
	moduleSimpleCommandExecRoutine proc;
} KviKvsModuleSimpleCommandExecRoutine;

typedef bool (*moduleFunctionExecRoutine)(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsVariant * pRetBuffer);
typedef struct _KviKvsModuleFunctionExecRoutine
{
	moduleFunctionExecRoutine proc;
} KviKvsModuleFunctionExecRoutine;

typedef bool (*moduleCallbackCommandExecRoutine)(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsSwitchList * pSwitches,const KviKvsScript * pCallback);
typedef struct _KviKvsModuleCallbackCommandExecRoutine
{
	moduleCallbackCommandExecRoutine proc;
} KviKvsModuleCallbackCommandExecRoutine;


class KVIRC_API KviKvsModuleInterface
{
public:
	KviKvsModuleInterface();
	~KviKvsModuleInterface();
protected:
	QDict<KviKvsModuleSimpleCommandExecRoutine>         * m_pModuleSimpleCommandExecRoutineDict;
	QDict<KviKvsModuleFunctionExecRoutine>              * m_pModuleFunctionExecRoutineDict;
	QDict<KviKvsModuleCallbackCommandExecRoutine>       * m_pModuleCallbackCommandExecRoutineDict;
public:
	void registerKvsSimpleCommand(const QString &szCommand,KviKvsModuleSimpleCommandExecRoutine *r)
		{ m_pModuleSimpleCommandExecRoutineDict->replace(szCommand,r); };
	void registerKvsCallbackCommand(const QString &szCommand,KviKvsModuleCallbackCommandExecRoutine *r)
		{ m_pModuleCallbackCommandExecRoutineDict->replace(szCommand,r); };
	void registerKvsFunction(const QString &szFunction,KviKvsModuleFunctionExecRoutine *r)
		{ m_pModuleFunctionExecRoutineDict->replace(szFunction,r); };

	void unregisterKvsSimpleCommand(const QString &szCommand)
		{ m_pModuleSimpleCommandExecRoutineDict->remove(szCommand); };
	void unregisterKvsCallbackCommand(const QString &szCommand)
		{ m_pModuleCallbackCommandExecRoutineDict->remove(szCommand); };
	void unregisterKvsFunction(const QString &szFunction)
		{ m_pModuleFunctionExecRoutineDict->remove(szFunction); };

	void unregisterAllKvsSimpleCommands()
		{ m_pModuleSimpleCommandExecRoutineDict->clear(); };
	void unregisterAllKvsCallbackCommands()
		{ m_pModuleCallbackCommandExecRoutineDict->clear(); };
	void unregisterAllKvsFunctions()
		{ m_pModuleFunctionExecRoutineDict->clear(); };

	KviKvsModuleSimpleCommandExecRoutine * findKvsSimpleCommand(const QString &szCommand)
		{ return m_pModuleSimpleCommandExecRoutineDict->find(szCommand); };
	KviKvsModuleCallbackCommandExecRoutine * findKvsCallbackCommand(const QString &szCommand)
		{ return m_pModuleCallbackCommandExecRoutineDict->find(szCommand); };
	KviKvsModuleFunctionExecRoutine * findKvsFunction(const QString &szFunction)
		{ return m_pModuleFunctionExecRoutineDict->find(szFunction); };
};








#define KVSM_SIMPLE_COMMAND(_proc) \
	static bool _proc(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsSwitchList * pSwitches)

#define KVSM_CALLBACK_COMMAND(_proc) \
	static bool _proc(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsSwitchList * pSwitches,const KviKvsScript * pCallback)

#define KVSM_FUNCTION(_proc) \
	static bool _proc(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsVariant * pResult)


#define KVSM_REGISTER_SIMPLE_COMMAND(_pModule,_szCmd,_procname) \
	{ \
		KviKvsModuleSimpleCommandExecRoutine * r = new KviKvsModuleSimpleCommandExecRoutine; \
		r->proc = _procname; \
		_pModule->registerKvsSimpleCommand(_szCmd,r); \
	}

#define KVSM_UNREGISTER_SIMPLE_COMMAND(_pModule,_szCmd) \
	_pModule->unregisterKvsSimpleCommand(_szCmd);

#define KVSM_REGISTER_CALLBACK_COMMAND(_pModule,_szCmd,_procname) \
	{ \
		KviKvsModuleCallbackCommandExecRoutine * r = new KviKvsModuleCallbackCommandExecRoutine; \
		r->proc = _procname; \
		_pModule->registerKvsCallbackCommand(_szCmd,r); \
	}

#define KVSM_UNREGISTER_CALLBACK_COMMAND(_pModule,_szCmd) \
	_pModule->unregistereKvsCallbackCommand(_szCmd);

#define KVSM_REGISTER_FUNCTION(_pModule,_szFnc,_procname) \
	{ \
		KviKvsModuleFunctionExecRoutine * r = new KviKvsModuleFunctionExecRoutine; \
		r->proc = _procname; \
		_pModule->registerKvsFunction(_szCmd,r); \
	}

#define KVSM_UNREGISTER_FUNCTION(_pModule,_szFnc) \
	_pModule->unregisterKvsFunction(_szFnc);


#define KVSM_UNREGISTER_ALL_SIMPLE_COMMANDS(_pModule) \
	_pModule->unregisterAllKvsSimpleCommands();

#define KVSM_UNREGISTER_ALL_CALLBACK_COMMANDS(_pModule) \
	_pModule->unregisterAllKvsCallbackCommands();

#define KVSM_UNREGISTER_ALL_FUNCTIONS(_pModule) \
	_pModule->unregisterAllKvsFunctions();








/*
class KviKvsModuleRunTimeCall
{
protected:
	KviKvsRunTimeContext  * m_pContext;
	KviKvsModuleInterface * m_pModule;
	KviKvsVariantList     * m_pParams;
public:
	KviKvsModuleRunTimeCall(KviKvsModuleInterface * pModule,
					KviKvsRunTimeContext * pContext,
					KviKvsVariantList * pParams)
		: m_pModule(pModule),m_pContext(pContext),m_pParams(pParams){};
	~KviKvsModuleRunTimeCall(){};				
public:
	KviKvsModuleInterface * module(){ return m_pModule; };
	KviKvsRunTimeContext * context(){ return m_pContext; };

	KviKvsParameterList * params(){ return m_pParams; };
	KviKvsParameterList * parameterList(){ return m_pParams; };

	KviKvsSwitchList * switches(){ return m_pSwitchList; };
	KviKvsSwitchList * switchList(){ return m_pSwitchList; };

	unsigned int parameterCount(){ return m_pParams->count(); };
	unsigned int paramCount(){ return m_pParams->count(); };

	KviKvsVariant * firstParam(){ return m_pParams->first(); };
	KviKvsVariant * nextParam(){ return m_pParams->next(); };

};

class KviKvsModuleFunctionCall
{
	friend class KviKvsTreeNodeModuleFunctionCall;
protected:
	KviKvsVariant * m_pResult;
protected:
	KviKvsModuleFunctionCall(KviKvsModuleInterface * pModule,
					KviKvsRunTimeContext * pContext,
					KviKvsVariantList * pParams,
					KviKvsVariant * pResult)
		: KviKvsModuleRunTimeCall(pModule,pContext,pParams), m_pResult(pResult)
		{};
	~KviKvsModuleFunctionCall(){};
public:
	void setResult(const QString &szValue){ m_pResult->setString(szValue); };
	void setResult(int iValue){ m_pResult->setInteger(iValue); };
	void setResult(double dValue){ m_pResult->setReal(dValue); };
	void setNullResult(){ m_pResult->setNothing(); };
	KviKvsVariant * result(){ return m_pResult; };
}

KviKvsModuleFunctionCall call(c,m,pParams,pRetVal);
*/

#endif //!_KVI_KVS_MODULEINTERFACE_H_
