#ifndef _OPTW_HIGHLIGHTING_CPP_
#define _OPTW_HIGHLIGHTING_CPP_
//
//   File : optw_highlighting.cpp
//   Creation date : Sat Nov  3 18:09:35 2001 GMT by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2001 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "optw_highlighting.h"

#include <qlayout.h>

#include "kvi_options.h"
#include "kvi_locale.h"

#include <qlabel.h>

#ifdef COMPILE_INFO_TIPS
    #include <qtooltip.h>
#endif

KviHighlightingOptionsWidget::KviHighlightingOptionsWidget(QWidget * parent)
: KviOptionsWidget(parent,"highlighting_options_widget")
{
	createLayout(8,1);

	connect(
		m_pUseWordHighlighting = addBoolSelector(0,0,0,0,__tr("Enable word highlighting"),KviOption_boolUseWordHighlighting),
		SIGNAL(toggled(bool)),
		addStringListSelector(0,1,0,1,__tr("Highlight words"),KviOption_stringlistHighlightWords,KVI_OPTION_BOOL(KviOption_boolUseWordHighlighting)),
		SLOT(setEnabled(bool)));
	// This can be used even without Word highlighting
	m_pAlwaysHighlightNick = addBoolSelector(0,2,0,2,__tr("Always highlight messages with my nick on them"),KviOption_boolAlwaysHighlightNick);
       KviBoolSelector * b3 = addBoolSelector(0,3,0,3,__tr("Hightlight only at normal msg"),KviOption_boolHighlightOnlyNormalMsg);
#ifdef COMPILE_INFO_TIPS
		 QToolTip::add(b3, __tr2qs("<center>If this option is enabled, kvirc will highlight the taskbar button only if a normal message in channel window shown. Actions like join, part and mode changes will be not highlighted. This is useful if you are in channels with high rate of join, part and/or mode changes and only the messages are interesting for you.</center>"));
#endif
   KviBoolSelector * b4 = addBoolSelector(0,4,0,4,__tr("Highlight at highlight msg too"),KviOption_boolHighlightOnlyNormalMsgHighlightInChanToo,KVI_OPTION_BOOL(KviOption_boolHighlightOnlyNormalMsg));
#ifdef COMPILE_INFO_TIPS
	QToolTip::add(b4, __tr2qs("<center>If this option is enabled, kvirc will also highlight the taskbar button on messages which contain a word from the <b>Highlight words</b> option above.</center>"));
#endif
   KviBoolSelector * b5 = addBoolSelector(0,5,0,5,__tr("Highlight at query msg too"),KviOption_boolHighlightOnlyNormalMsgQueryToo,KVI_OPTION_BOOL(KviOption_boolHighlightOnlyNormalMsg));
#ifdef COMPILE_INFO_TIPS
	QToolTip::add(b5, __tr2qs("<center>If this option is enabled, KVIrc will also highlight the taskbar button on messages which are shown in queries.</center>"));
#endif
   connect(b3,SIGNAL(toggled(bool)),b4,SLOT(setEnabled(bool)));
   connect(b3,SIGNAL(toggled(bool)),b5,SLOT(setEnabled(bool)));
   KviBoolSelector * b6 = addBoolSelector(0,6,0,6,__tr("Highlight only at costum alert level reached"),KviOption_boolHighlightOnlyAtCostumHighlightLevel);
#ifdef COMPILE_INFO_TIPS
	QToolTip::add(b6, __tr2qs("<center>If this option is enabled, KVIrc will only highlight if the costum alert level is reached.</center>"));
#endif
   KviUIntSelector * b7 = addUIntSelector(0,7,0,7,__tr("Minimal alert level"),KviOption_uintMinHighlightLevel,1,0xFFFFFFFF,KVI_MSGTYPE_MAXLEVEL,KVI_OPTION_UINT(KviOption_boolHighlightOnlyAtCostumHighlightLevel));
   b7->setEnabled(KviOption_boolHighlightOnlyAtCostumHighlightLevel);
#ifdef COMPILE_INFO_TIPS
	QToolTip::add(b6, __tr2qs("<center>This option sets the minimal alert level at which will be highlighted.</center>"));
#endif
   connect(b6,SIGNAL(toggled(bool)),b7,SLOT(setEnabled(bool)));
   addLabel(0,8,0,8," ",true);

	layout()->setRowStretch(2,2);

}

KviHighlightingOptionsWidget::~KviHighlightingOptionsWidget()
{
}


#include "m_optw_highlighting.moc"

#endif //_OPTW_HIGHLIGHTING_CPP_
