//
//   File : optw_identity.cpp
//   Creation date : Tue Nov 14 2000 23:06:53 CEST by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include "optw_identity.h"

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qgroupbox.h>
#include <qtimer.h>

#include "kvi_options.h"
#include "kvi_locale.h"
#include "kvi_uparser.h" 
#include "kvi_app.h"
#include "kvi_window.h"
#include "kvi_console.h"
#include "kvi_optionswidget.h"
#include "kvi_filedialog.h"
#include "kvi_iconmanager.h"
#include "kvi_http.h"
#include "kvi_avatar.h"

//#warning "Info tips"


KviAvatarDownloadDialog::KviAvatarDownloadDialog(QWidget * par,const QString &szUrl)
: QDialog(par)
{
	setCaption(__tr2qs("Avatar Download - KVIrc"));

	m_szUrl = szUrl;

	QGridLayout * g = new QGridLayout(this,2,2,4,4);

	m_pOutput = new QLabel(__tr2qs("<center>Please wait while the avatar is being downloaded</center>"),this);
	g->addMultiCellWidget(m_pOutput,0,0,0,1);

	QPushButton * b = new QPushButton(__tr2qs("Abort"),this);
	g->addWidget(b,1,1);
	connect(b,SIGNAL(clicked()),this,SLOT(cancelClicked()));

	m_pRequest = new KviHttpRequest();
	
	QTimer::singleShot(0,this,SLOT(startDownload()));

	g->setRowStretch(0,1);
	g->setColStretch(0,1);

	setMinimumSize(250,120);
}

KviAvatarDownloadDialog::~KviAvatarDownloadDialog()
{
	delete m_pRequest;
}

void KviAvatarDownloadDialog::startDownload()
{
	connect(m_pRequest,SIGNAL(terminated(bool)),this,SLOT(downloadTerminated(bool)));
	connect(m_pRequest,SIGNAL(status(const char *)),this,SLOT(downloadMessage(const char *)));

	QString tmp = m_szUrl;
	g_pIconManager->urlToCachedFileName(tmp);
	g_pApp->getLocalKvircDirectory(m_szLocalFileName,KviApp::Avatars,tmp);

	if(!m_pRequest->get(KviUrl(m_szUrl),KviHttpRequest::StoreToFile,m_szLocalFileName.latin1()))
	{
		m_szErrorMessage = __tr2qs("Failed to start the download");
		reject();
	}
}

void KviAvatarDownloadDialog::closeEvent(QCloseEvent * e)
{
	m_szErrorMessage = __tr2qs("Download aborted by user");
	e->ignore();
	reject();
}

void KviAvatarDownloadDialog::cancelClicked()
{
	m_szErrorMessage = __tr2qs("Download aborted by user");
	reject();
}


void KviAvatarDownloadDialog::downloadMessage(const char * message)
{
	if(message)
	{
		QString txt = "<center>";
		txt += message;
		txt += "</center>";
		m_pOutput->setText(message);
	}
}


void KviAvatarDownloadDialog::downloadTerminated(bool bSuccess)
{
	if(bSuccess)
	{
		accept();
	} else {
		m_szErrorMessage = m_pRequest->lastError().ptr();
		reject();
	}
}



KviAvatarSelectionDialog::KviAvatarSelectionDialog(QWidget * par,const QString &szInitialPath)
: QDialog(par)
{
	setCaption(__tr2qs("Choose an Avatar - KVIrc"));

	QGridLayout * g = new QGridLayout(this,4,3,4,4);
	
	QLabel * l = new QLabel(__tr2qs("<center>Please select an avatar image.</center>" \
				"The full path to a local file or an image on the Web can be used.<br>" \
				"If you wish to use a local image file, click " \
				"the \"<b>Browse</b>\" button to browse local folders.<br>" \
				"The full URL for an image (including <tt>http://</tt>) can be entered manually.<br>" \
				),this);

	g->addMultiCellWidget(l,0,0,0,2);
	
	m_pLineEdit = new QLineEdit(this);
	m_pLineEdit->setText(szInitialPath);
	g->addMultiCellWidget(m_pLineEdit,1,1,0,1);

	QPushButton * b = new QPushButton(__tr2qs("&Browse..."),this);
	connect(b,SIGNAL(clicked()),this,SLOT(chooseFileClicked()));
	g->addWidget(b,1,2);

	b = new QPushButton(__tr2qs("&OK"),this);
	b->setDefault(true);
	connect(b,SIGNAL(clicked()),this,SLOT(okClicked()));
	g->addWidget(b,3,1);

	b = new QPushButton(__tr2qs("Cancel"),this);
	connect(b,SIGNAL(clicked()),this,SLOT(cancelClicked()));
	g->addWidget(b,3,2);
	
	//g->setRowSpacing(2,15);
	g->setRowStretch(0,1);
	g->setColStretch(0,1);
}

KviAvatarSelectionDialog::~KviAvatarSelectionDialog()
{
}

void KviAvatarSelectionDialog::okClicked()
{
	m_szAvatarName = m_pLineEdit->text();
	accept();
}

void KviAvatarSelectionDialog::cancelClicked()
{
	reject();
}

void KviAvatarSelectionDialog::chooseFileClicked()
{
	QString tmp;
 	if(KviFileDialog::askForOpenFileName(tmp,__tr2qs("Choose an Image File - KVIrc")))
	{
		m_pLineEdit->setText(tmp);
	}
}

void KviAvatarSelectionDialog::closeEvent(QCloseEvent * e)
{
	e->ignore();
	reject();
}



KviIdentityOptionsWidget::KviIdentityOptionsWidget(QWidget * parent)
: KviOptionsWidget(parent,"identity_options_widget")
{
	m_iLastEditedNick = 0;
	m_NicksTable = new KviStr[4];
	int i = 0;
	m_NicksTable[i] = KviStr(KVI_OPTION_STRING(KviOption_stringNickname1));
	m_NicksTable[i++] = KviStr(KVI_OPTION_STRING(KviOption_stringNickname2));
	m_NicksTable[i++] = KviStr(KVI_OPTION_STRING(KviOption_stringNickname3));
	m_NicksTable[i++] = KviStr(KVI_OPTION_STRING(KviOption_stringNickname4));

	m_sModeStr = KVI_OPTION_STRING(KviOption_stringDefaultUserMode);    
	m_bI = m_sModeStr.contains('i',false);
	m_bW = m_sModeStr.contains('w',false);
	m_bS = m_sModeStr.contains('s',false);   
    
	createLayout(3,1);

	QGroupBox * gbox = new QGroupBox(1,QGroupBox::Horizontal,__tr("Properties" ),this);
	QHBox * hb = new QHBox(gbox);
	hb->setSpacing(4);
	m_pNickBox = new QComboBox(false,hb);
	m_pNickBox->insertItem(__tr2qs("Nickname"));
	m_pNickBox->insertItem(__tr2qs("Nickname 2"));
	m_pNickBox->insertItem(__tr2qs("Nickname 3"));
	m_pNickBox->insertItem(__tr2qs("Nickname 4"));
	m_pNickEdit = new QLineEdit(hb);
	m_pChangeButton = new QPushButton(__tr2qs("&Change"),hb);

	addStringSelectorNoLayout(gbox,__tr2qs("Username:"),KviOption_stringUsername);
	addStringSelectorNoLayout(gbox,__tr2qs("Real name:"),KviOption_stringRealname);

	m_pNickBox->setCurrentItem(m_iLastEditedNick);
	m_pNickEdit->setText(m_NicksTable[m_iLastEditedNick].ptr());
	connect(m_pNickBox,SIGNAL(activated(int)),this,SLOT(selectNick(int)));
	addWidgetToLayout(gbox,0,0,0,0);

	gbox = new QGroupBox(1,QGroupBox::Horizontal,__tr("User Mode" ),this);
	m_pISelector = addBoolSelectorNoLayout(gbox,__tr2qs("Invisible (+i)"),m_bI);
	m_pSSelector = addBoolSelectorNoLayout(gbox,__tr2qs("Server notices (+s)"),m_bS);
	m_pWSelector = addBoolSelectorNoLayout(gbox,__tr2qs("Wallops (+w)"),m_bW);

	bool b_Iscon = (g_pApp->activeWindow()->console() ? g_pApp->activeWindow()->console()->isConnected() : false);
	m_pChangeButton->setEnabled(b_Iscon);

	connect(m_pChangeButton,SIGNAL(clicked()),this,SLOT(changeNick()));
    
	connect(m_pISelector,SIGNAL(toggled(bool)),this,SLOT(iModeCheckToggled(bool)));
	connect(m_pWSelector,SIGNAL(toggled(bool)),this,SLOT(wModeCheckToggled(bool)));
	connect(m_pSSelector,SIGNAL(toggled(bool)),this,SLOT(sModeCheckToggled(bool)));
	addWidgetToLayout(gbox,0,1,0,1);
    
	// AVATAR

	m_pLocalAvatar = new KviPixmap(KVI_OPTION_PIXMAP(KviOption_pixmapMyAvatar));

	bool bHaveAvatar = (!KVI_OPTION_STRING(KviOption_stringMyAvatar).isEmpty()) && m_pLocalAvatar->pixmap();

	gbox = new QGroupBox(1,QGroupBox::Horizontal,__tr("Avatar" ),this);

	m_pUseAvatarCheck = new QCheckBox(__tr2qs("Use avatar"),gbox);
	m_pUseAvatarCheck->setChecked(bHaveAvatar);

	m_pAvatarPreview = new KviPixmapPreview(gbox);
	m_pAvatarPreview->setPixmap(m_pLocalAvatar);
	m_pAvatarPreview->setEnabled(bHaveAvatar);
	connect(m_pUseAvatarCheck,SIGNAL(toggled(bool)),m_pAvatarPreview,SLOT(setEnabled(bool)));

	hb = new QHBox(gbox);
	hb->setSpacing(4);

	m_pAvatarNameEdit = new QLineEdit(hb);
	m_pAvatarNameEdit->setReadOnly(true);
	m_pAvatarNameEdit->setEnabled(bHaveAvatar);
	m_pAvatarNameEdit->setText(KVI_OPTION_STRING(KviOption_stringMyAvatar).ptr());
	connect(m_pUseAvatarCheck,SIGNAL(toggled(bool)),m_pAvatarNameEdit,SLOT(setEnabled(bool)));

	m_pChooseAvatarButton = new QPushButton(__tr2qs("Choose..."),hb);
	m_pChooseAvatarButton->setEnabled(bHaveAvatar);
	connect(m_pUseAvatarCheck,SIGNAL(toggled(bool)),m_pChooseAvatarButton,SLOT(setEnabled(bool)));

	connect(m_pChooseAvatarButton,SIGNAL(clicked()),this,SLOT(chooseAvatar()));

	addWidgetToLayout(gbox,0,2,0,2);

	//layout()->setRowStretch(12,1);
	//layout()->setColStretch(0,1);
}

KviIdentityOptionsWidget::~KviIdentityOptionsWidget()
{
	delete[] m_NicksTable;
	delete m_pLocalAvatar;
}

void KviIdentityOptionsWidget::commit(void)
{
	KviOptionsWidget::commit();

	int i = 0;
	KviStr nick = m_pNickEdit->text();
	if(!nick.isEmpty())m_NicksTable[m_iLastEditedNick] = nick;
	KVI_OPTION_STRING(KviOption_stringNickname1) = m_NicksTable[i];
	KVI_OPTION_STRING(KviOption_stringNickname2) = m_NicksTable[i++];
	KVI_OPTION_STRING(KviOption_stringNickname3) = m_NicksTable[i++];
	KVI_OPTION_STRING(KviOption_stringNickname4) = m_NicksTable[i++];

	KVI_OPTION_STRING(KviOption_stringDefaultUserMode) = m_sModeStr;

	QString szAvatarName = m_pAvatarNameEdit->text();

	if(m_pLocalAvatar->pixmap() && (!szAvatarName.isEmpty()))
	{
		KVI_OPTION_STRING(KviOption_stringMyAvatar) = szAvatarName;
		KVI_OPTION_PIXMAP(KviOption_pixmapMyAvatar) = *m_pLocalAvatar;
	} else {
		KVI_OPTION_STRING(KviOption_stringMyAvatar) = "";
		KVI_OPTION_PIXMAP(KviOption_pixmapMyAvatar) = KviPixmap();
	}
}

void KviIdentityOptionsWidget::chooseAvatar()
{
	QString szCurrent = m_pAvatarNameEdit->text();

	KviAvatarSelectionDialog dlg(this,szCurrent);
	if(dlg.exec() != QDialog::Accepted)return;

	szCurrent = dlg.avatarName();
	szCurrent.stripWhiteSpace();

	if(szCurrent.startsWith(QString("http://")) && (szCurrent.length() > 7))
	{
		// this is an url
		// first check if we have it in the cache
		KviAvatar * a = g_pIconManager->getAvatar(szCurrent.latin1());
		if(!a)
		{
			// no cache... try downloading
			KviAvatarDownloadDialog dlg(this,szCurrent);
			if(dlg.exec() == QDialog::Accepted)
			{
				// downloaded!
				// try to load it
				if(!m_pLocalAvatar->load(dlg.localFileName().latin1()))
				{
					// unloadable
					szCurrent = "";
					QMessageBox::warning(this,__tr2qs("Failed to Load Avatar - KVIrc"),
						__tr2qs("Failed to load the avatar image.<br>" \
							"It may be an unaccessible file or an<br>" \
							"unsupported image format."),QMessageBox::Ok,QMessageBox::NoButton);
				} // else loaded!
			} else {
				// unloadable
				QString tmp;
				KviQString::sprintf(tmp,__tr2qs("Failed to download the avatar image.<br>" \
					"<b>%Q</b>"),&(dlg.errorMessage()));
				QMessageBox::warning(this,__tr2qs("Avatar Download Failed - KVIrc"),tmp,QMessageBox::Ok,QMessageBox::NoButton);
				szCurrent = "";
				m_pLocalAvatar->setNull();
			}
		} else {
			// else OK.. got it in the cache
			m_pLocalAvatar->set(*(a->pixmap()),a->localPath());
			delete a;
		}
	} else {
		// this is a local path
		if(m_pLocalAvatar->load(szCurrent.latin1()))
		{
			// local path, loaded
			QString tmp = szCurrent;
			int idx = tmp.findRev("/");
			if(idx != -1)
			{
				szCurrent = tmp.right(tmp.length() - (idx + 1));
				tmp = szCurrent;
			}
			idx = tmp.findRev("\\");
			if(idx != -1)
			{
				szCurrent = tmp.right(tmp.length() - (idx + 1));
				tmp = szCurrent;
			}
		} else {
			// unloadable
			QMessageBox::warning(this,__tr2qs("Failed to Load Avatar - KVIrc"),
					__tr2qs("Failed to load the avatar image.<br>" \
						"It may be an unaccessible file or an<br>" \
						"unsupported image format."),QMessageBox::Ok,QMessageBox::NoButton);
			szCurrent = "";
		}
	}

	m_pAvatarNameEdit->setText(szCurrent);
	m_pAvatarPreview->setPixmap(m_pLocalAvatar);
}

void KviIdentityOptionsWidget::iModeCheckToggled(bool){changeStatus('i');}
void KviIdentityOptionsWidget::wModeCheckToggled(bool){changeStatus('w');}
void KviIdentityOptionsWidget::sModeCheckToggled(bool){changeStatus('s');}

void KviIdentityOptionsWidget::changeStatus(char c)
{    
    KviStr tmp(m_sModeStr);
    KviStr tmp2(m_sModeStr);
    
    bool iToggled = m_pISelector->isChecked();
    bool wToggled = m_pWSelector->isChecked();
    bool sToggled = m_pSSelector->isChecked();
    bool actualCheck;
    
    switch(c)
    {
        case 'i': actualCheck = iToggled;break;
        case 'w': actualCheck = wToggled;break;
        case 's': actualCheck = sToggled;break;
    }
    
    if(actualCheck && !m_sModeStr.contains(c))m_sModeStr.append(c);
    else if(m_sModeStr.contains(c))
    { // Remove the char let's hope there is no more than ONE ocurrence of it (it SHOULDN'T)    
        m_sModeStr = tmp.cutToFirst(c);   
        m_sModeStr.append(tmp2.cutFromFirst(c));    
    }     
}        

void KviIdentityOptionsWidget::butModeClicked(void){}

void KviIdentityOptionsWidget::selectNick(int i)
{
	if(i != -1)
	{
		KviStr nick = m_pNickEdit->text();
		if(!nick.isEmpty())m_NicksTable[m_iLastEditedNick] = nick;

		m_iLastEditedNick = i;

		m_pNickEdit->setText(m_NicksTable[i].ptr());
	}
}

void KviIdentityOptionsWidget::changeNick()
{
	KviStr nick = m_pNickEdit->text();
	if(!nick.isEmpty())
	{
		KviStr command;
		command.sprintf("nick %s",nick.ptr());
		g_pUserParser->parseCommandBuffer(command.ptr(),g_pApp->activeWindow());
	}
}



#include "m_optw_identity.moc"
