//
//   File : optw_servers.cpp
//   Creation date : Wed Jun 12 2000 03:29:51 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "optw_servers.h"
#include "kvi_query.h"
#include "kvi_channel.h"
#include "kvi_locale.h"
#include "kvi_iconmanager.h"
#include "kvi_ircserverdb.h"
#include "kvi_ipeditor.h"
#include "kvi_netutils.h"
#include "kvi_settings.h"
#include "kvi_filedialog.h"
#include "kvi_config.h"
#include "kvi_scripteditor.h"
#include "kvi_options.h"
#include "kvi_uparser.h"
#include "kvi_app.h"
#include "kvi_console.h"
#include "kvi_ircurl.h"
#include "kvi_module.h"
#include "kvi_modulemanager.h"
#include "kvi_moduleextension.h"
#include "kvi_msgbox.h"
#include "kvi_mexserverimport.h"

#include <qtoolbutton.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <qhbox.h>
#include <qasciidict.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qinputdialog.h>

KviServerDetailsWidget::KviServerDetailsWidget(QWidget * par,KviIrcServer * s)
    : QDialog(par,"server_details",true)
{
  server = s;
  QGridLayout * g = new QGridLayout(this,4,4,10,5);

  setCaption(__tr2qs("Server Details"));
  setIcon(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVER)));

  QString szTmp = "<font size=\"+1\"><b>irc";
  if(s->isIpV6())szTmp += "6";
  szTmp += "://";
  szTmp += s->m_szHostname.ptr();
  szTmp += ":";

  QString num;
  num.setNum(s->port());
  szTmp += num;
  szTmp += "</b></font>";

  QLabel * l = new QLabel(szTmp,this);
  l->setFrameStyle(QFrame::Raised | QFrame::StyledPanel);
  l->setAlignment(AlignCenter);
  l->setMargin(10);
  g->addMultiCellWidget(l,0,0,0,3);
  l->setMinimumWidth(200);


  l = new QLabel(__tr2qs("Description:"),this);
  g->addWidget(l,1,0);
  m_pDescEditor = new QLineEdit(this);
  g->addMultiCellWidget(m_pDescEditor,1,1,1,3);
  m_pDescEditor->setText(s->description());
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pDescEditor,__tr2qs("<center>This is a brief description of the server. " \
                                        "This field has no restrictions but many server lists use it to describe the server's physical location</center>"));
#endif


  // Identity tab

  QTabWidget * tw = new QTabWidget(this);
  g->addMultiCellWidget(tw,2,2,0,3);

  QWidget * tab = new QWidget(tw);
  QGridLayout * gl = new QGridLayout(tab,10,2,10,4);

  l = new QLabel(__tr2qs("Username:"),tab);
  gl->addWidget(l,0,0);
  m_pUserEditor = new QLineEdit(tab);
  gl->addWidget(m_pUserEditor,0,1);
  m_pUserEditor->setText(s->user());
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pUserEditor,__tr2qs("<center>This is the <b>username</b> that KVIrc will use to login to this server.\n" \
                                        "If this field is left empty (most common case), KVIrc will use the default username " \
                                        "specified in the \"Identity\" options tab.</center>"));
#endif

  l = new QLabel(__tr2qs("Password:"),tab);
  gl->addWidget(l,1,0);
  m_pPassEditor = new QLineEdit(tab);
  m_pPassEditor->setEchoMode(QLineEdit::Password);
  gl->addWidget(m_pPassEditor,1,1);
  m_pPassEditor->setText(s->pass());
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pPassEditor,__tr2qs("<center>If the server requires a password, put it in this field, otherwise leave it empty.</center>"));
#endif

  l = new QLabel(__tr2qs("Nickname:"),tab);
  gl->addWidget(l,2,0);
  m_pNickEditor = new QLineEdit(tab);
  gl->addWidget(m_pNickEditor,2,1);
  m_pNickEditor->setText(s->nick());
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pNickEditor,__tr2qs("<center>You can specify a \"special\" <b>nickname</b> that can be used to login to this server.<br>" \
                                        "If this field is left empty (most common case), the default nickname (specified in the \"Identity\" tab) will be used.</center>"));
#endif

  l = new QLabel(__tr2qs("Real name:"),tab);
  gl->addWidget(l,3,0);
  m_pRealEditor = new QLineEdit(tab);
  gl->addWidget(m_pRealEditor,3,1);
  m_pRealEditor->setText(s->realName());
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pRealEditor,__tr2qs("<center>You can specify here a \"special\" <b>real name</b> that can be used to login with this server.<br>" \
                                        "If you leave this field empty (most common case), the default \"real name\" (specified in the \"Identity\" tab) will be used.</center>"));
#endif


  QFrame * f = new QFrame(tab);
  f->setFrameStyle(QFrame::HLine |QFrame::Sunken);

  gl->addMultiCellWidget(f,4,4,0,1);


  m_pUseDefaultInitUMode = new QCheckBox(__tr2qs("Use default initial user mode"),tab);
  gl->addMultiCellWidget(m_pUseDefaultInitUMode,5,5,0,2);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pUseDefaultInitUMode,__tr2qs("<center>If this is enabled, the global initial <b>user mode</b> (configured from"\
                  " the identity dialog) will be used. If disabled, you can configure an initial user mode for this server"));
#endif
  m_pUseDefaultInitUMode->setChecked(s->initUMode().hasData());
  connect(m_pUseDefaultInitUMode,SIGNAL(toggled(bool)),this,SLOT(useDefaultInitUModeToggled(bool)));

  m_pIMode = new QCheckBox(__tr2qs("Invisible (+i)"),tab);
  m_pIMode->setEnabled(!m_pUseDefaultInitUMode->isChecked());
  m_pIMode->setChecked(s->initUMode().contains('i'));
  gl->addMultiCellWidget(m_pIMode,6,6,0,1);

  m_pWMode = new QCheckBox(__tr2qs("Wallops (+w)"),tab);
  m_pWMode->setEnabled(!m_pUseDefaultInitUMode->isChecked());
  m_pWMode->setChecked(s->initUMode().contains('w'));
  gl->addMultiCellWidget(m_pWMode,7,7,0,1);

  m_pSMode = new QCheckBox(__tr2qs("Server notices (+s)"),tab);
  m_pSMode->setEnabled(!m_pUseDefaultInitUMode->isChecked());
  m_pSMode->setChecked(s->initUMode().contains('s'));
  gl->addMultiCellWidget(m_pSMode,8,8,0,1);

  l = new QLabel("",tab);
  gl->addMultiCellWidget(l,9,9,0,1);

  gl->setRowStretch(9,1);
  gl->setColStretch(1,1);


  tw->addTab(tab,__tr2qs("Identity"));



  // Connection tab

  tab = new QWidget(tw);
  gl = new QGridLayout(tab,7,2,10,4);


  l = new QLabel(__tr2qs("IP address:"),tab);
  gl->addWidget(l,0,0);
  m_pIpEditor = new KviIpEditor(tab,KviIpEditor::IpV4);
  gl->addWidget(m_pIpEditor,0,1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pIpEditor,__tr2qs("<center>This is the <b>IP address</b> of the server, it is here for caching purposes.<br>" \
                                      "If you leave this field empty, KVIrc will fill it the first time that it connects to the server. " \
                                      "If you enable the \"cache IP address\" option below, KVIrc will use it as a \"cached result\" " \
                                      "and avoid looking it up again.</center>"));
#endif
#ifdef COMPILE_IPV6_SUPPORT
  m_pIpEditor->setAddressType(s->isIpV6() ? KviIpEditor::IpV6 : KviIpEditor::IpV4);
#else
  m_pIpEditor->setAddressType(KviIpEditor::IpV4);
#endif

  if(!m_pIpEditor->setAddress(s->m_szIp.ptr()))
  {
#ifdef COMPILE_IPV6_SUPPORT
    m_pIpEditor->setAddress(s->isIpV6() ? "0:0:0:0:0:0:0:0" : "0.0.0.0");
#else
    m_pIpEditor->setAddress("0.0.0.0");
#endif

  }

  m_pCacheIpCheck = new QCheckBox(__tr2qs("Cache IP address"),tab);
  gl->addMultiCellWidget(m_pCacheIpCheck,1,1,0,1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pCacheIpCheck,__tr2qs("<center>This check will enable <b>IP address caching</b> for this server:<br>" \
                                          "DNS lookups can be time consuming and might be blocking on some platforms; " \
                                          "this option will cause KVIrc to look up the server hostname only once.</center>"));
#endif
  m_pCacheIpCheck->setChecked(s->cacheIp());

  f = new QFrame(tab);
  f->setFrameStyle(QFrame::HLine |QFrame::Sunken);

  gl->addMultiCellWidget(f,2,2,0,1);


  m_pUseSSLCheck = new QCheckBox(__tr2qs("Use SSL protocol"),tab);
  gl->addMultiCellWidget(m_pUseSSLCheck,3,3,0,1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pUseSSLCheck,__tr2qs("<center>This check will cause the connection to use the <b>Secure Socket Layer</b><br>" \
                                         "encryption support. Obviously, the server must have support for this too :)</center>"));
#endif
#ifndef COMPILE_SSL_SUPPORT
  m_pUseSSLCheck->setEnabled(false);
#endif
  m_pUseSSLCheck->setChecked(s->useSSL());

  f = new QFrame(tab);
  f->setFrameStyle(QFrame::HLine |QFrame::Sunken);

  gl->addMultiCellWidget(f,4,4,0,1);


  l = new QLabel(__tr2qs("Link filter:"),tab);
  gl->addWidget(l,5,0);
  m_pLinkFilterEditor = new QComboBox(true,tab);
  m_pLinkFilterEditor->setDuplicatesEnabled(false);
  gl->addWidget(m_pLinkFilterEditor,5,1);

  m_pLinkFilterEditor->insertItem("");

  g_pModuleManager->loadModulesByCaps("linkfilter");
  KviModuleExtensionDescriptorList * mexl = KviModuleExtensionManager::instance()->getExtensionList("linkfilter");



  if(mexl)
  {
    for(KviModuleExtensionDescriptor * d = mexl->first();d;d = mexl->next())
      m_pLinkFilterEditor->insertItem(d->name().ptr());
  }
  else
  {
    if(s->linkFilter().hasData())
      m_pLinkFilterEditor->insertItem(s->linkFilter().ptr());
  }

  if(s->linkFilter().hasData())
    m_pLinkFilterEditor->setCurrentText(s->linkFilter().ptr());
  else
    m_pLinkFilterEditor->setCurrentText("");


#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pLinkFilterEditor,__tr2qs("<center>This field specifies the name of a module that exports a link filter for this type of server.<br>" \
                  "For plain IRC connections you don't need any link filters; this is used for incompatible protocols.</center>"));
#endif

   f = new QFrame(tab);
  f->setFrameStyle(QFrame::HLine |QFrame::Sunken);

  gl->addMultiCellWidget(f,6,6,0,1);


  m_pUseAutoConnect = new QCheckBox(__tr2qs("Connect to this server at KVIrc startup"),tab);
  m_pUseAutoConnect->setChecked(s->m_pUseAutoConnect);

  //tmp
//  m_pUseAutoConnect->setEnabled(false);


  gl->addMultiCellWidget(m_pUseAutoConnect,7,7,0,1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pUseAutoConnect,__tr2qs("<center>This check will cause KVIrc to connect to the IRC server when it is started.</center>"));
#endif

  l = new QLabel("",tab);
  gl->addMultiCellWidget(l,8,8,0,1);

  gl->setRowStretch(12,1);
  gl->setColStretch(1,1);
  tw->addTab(tab,__tr2qs("Connection"));

  // after login join channels
  tab = new QWidget(tw);
  QBoxLayout *layout1 = new QBoxLayout (tab,QBoxLayout::Down);
  QScrollView *scroll_channel_join = new QScrollView(tab);
  QVBox*big_box = new QVBox(scroll_channel_join->viewport());
  channel_group = new QButtonGroup(__tr2qs("&Recent Channels"),big_box);
  channel_group->setRadioButtonExclusive ( false );

  QBoxLayout *buttonLayout = new QBoxLayout (channel_group,QBoxLayout::Down);

  // stored channels
  for(QStringList::Iterator it = s->m_pChannelList->begin(); it != s->m_pChannelList->end(); ++it)
  {
    QCheckBox *cb = new QCheckBox(*it , channel_group );
    buttonLayout->addWidget(cb);
    cb->setChecked( true );
  }

  // recent but not stored
  for(QStringList::Iterator it2 = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin(); it2 != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end(); ++it2)
  {
    bool insert=true;
    for (int ii=0;ii< channel_group->count();ii++)
        if (((QCheckBox*)(channel_group->find(ii)))->text() == *it2)
            insert =false;
    if (insert == true) {
        QCheckBox *cb = new QCheckBox( *it2, channel_group );
        buttonLayout->addWidget(cb);
        cb->setChecked( false );
    }
  }

  QPushButton *BtnNewChannel = new QPushButton("BtnNewChannel",tab);
  BtnNewChannel->setText(__tr("&Add Channel"));
  //connect(BtnNewChannel,SIGNAL(clicked()),this, SLOT(addNewChannel(tw, tab, s)));
	connect(BtnNewChannel,SIGNAL(clicked()),this, SLOT(addNewChannel()));

  channel_group->resize(tab->size());

  scroll_channel_join->addChild(channel_group);
  layout1->addWidget(scroll_channel_join);
  layout1->addWidget(BtnNewChannel);




  /*
      int i=0;
  	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin(); it != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end(); ++it)
  	{
          QCheckBox *tmp = new QCheckBox(*it,channel_group);
          tmp->setChecked(false);
          //QMessageBox::information(0,"neuer chan",*it);
          //channel_group->resize(channel_group->sizeHint());
  	}
  */

  /*
  #ifdef COMPILE_INFO_TIPS
  	QToolTip::add(m_pOnLoginEditor,__tr2qs("<center>The following commands will be executed after a succesfull login to this server.<br>" \
  				"This is useful for automatically joining channels, opening queries etc...</center>"));
  #endif
  */
  tw->addTab(tab,__tr2qs("Join Channels"));




  // after connect execute

  tab = new QWidget(tw);
  gl = new QGridLayout(tab,1,1,10,4);


  m_pOnConnectEditor = KviScriptEditor::createInstance(tab);
  gl->addWidget(m_pOnConnectEditor,0,0);
  m_pOnConnectEditor->setText(s->onConnectCommand());
  m_pOnConnectEditor->setMinimumHeight(150);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pOnConnectEditor,__tr2qs("<center>The following commands will be executed after the connection has been started.<br>" \
            "<b>Important:</b> enter the commands <b>without</b> the slash (for example: <i>quote pass secret</i> instead of <i>/quote pass secret</i>)<br>"\
            "KVIrc will first send the USER command, then eventually PASS and NICK and then execute this<br>" \
            "command sequence.<br>This is particularly useful with IRC bouncers.</center>"));
#endif


  tw->addTab(tab,__tr2qs("On Connect"));


  // after login execute

  tab = new QWidget(tw);
  gl = new QGridLayout(tab,1,1,10,4);

  m_pOnLoginEditor = KviScriptEditor::createInstance(tab);
  gl->addWidget(m_pOnLoginEditor,0,0);
  m_pOnLoginEditor->setText(s->onLoginCommand());

  m_pOnLoginEditor->setMinimumHeight(150);

#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pOnLoginEditor,__tr2qs("<center>The following commands will be executed after a succesfull login to this server.<br>" \
                                            "<b>Important:</b> enter the commands <b>without</b> the slash (for example: <i>join quote privatelog</i> instead of <i>/quote privatelog</i>)<br>"
                                           "This is useful for automatically joining channels, opening queries etc...</center>"));
#endif

  tw->addTab(tab,__tr2qs("On Login"));


  QPushButton * b = new QPushButton(__tr2qs("&OK"),this);
  b->setMinimumWidth(80);
  g->addWidget(b,3,2);
  b->setDefault(true);
  connect(b,SIGNAL(clicked()),this,SLOT(accept()));

  b = new QPushButton(__tr2qs("Cancel"),this);
  g->addWidget(b,3,3);
  b->setMinimumWidth(80);
  connect(b,SIGNAL(clicked()),this,SLOT(reject()));

  g->setRowStretch(2,1);
  g->setColStretch(1,1);

  setMinimumWidth(150);
}

KviServerDetailsWidget::~KviServerDetailsWidget()
{
  KviScriptEditor::destroyInstance(m_pOnConnectEditor);
  KviScriptEditor::destroyInstance(m_pOnLoginEditor);
}

void KviServerDetailsWidget::fillData(KviIrcServer * s)
{
  s->m_szUser = m_pUserEditor->text();
  s->m_szPass = m_pPassEditor->text();
  s->m_szNick = m_pNickEditor->text();
  s->m_szRealName = m_pRealEditor->text();
  s->m_szDescription = m_pDescEditor->text();
  s->m_szLinkFilter = m_pLinkFilterEditor->lineEdit()->text();
  s->m_szIp = "";

  s->setCacheIp(m_pCacheIpCheck->isChecked());
  s->setUseSSL(m_pUseSSLCheck->isChecked());

  KviStr szUMode;
  if(m_pUseDefaultInitUMode->isChecked())
  {
    if(m_pIMode->isChecked())szUMode.append('i');
    if(m_pWMode->isChecked())szUMode.append('w');
    if(m_pSMode->isChecked())szUMode.append('s');
  }
  s->setInitUMode(szUMode);

  KviStr tmpAddr = m_pIpEditor->address();

  if(!m_pIpEditor->hasEmptyFields())
  {
#ifdef COMPILE_IPV6_SUPPORT
    if(s->isIpV6())
    {
      if((!kvi_strEqualCI(tmpAddr.ptr(),"0:0:0:0:0:0:0:0")) && kvi_isValidStringIp_V6(tmpAddr.ptr()))
      {
        s->m_szIp = tmpAddr;
      }
      else
      {
        s->setCacheIp(false);
        s->m_szIp = "";
      }
    }
    else
    {
#endif
      if((!kvi_strEqualCI(tmpAddr.ptr(),"0.0.0.0")) && kvi_isValidStringIp(tmpAddr.ptr()))
      {
        s->m_szIp = tmpAddr;
      }
      else
      {
        s->setCacheIp(false);
        s->m_szIp = "";
      }
#ifdef COMPILE_IPV6_SUPPORT

    }
#endif

  }
  else
  {
    s->setCacheIp(false);
    s->m_szIp = "";
  }

  QString tmp;
  m_pOnConnectEditor->getText(tmp);
  s->m_szOnConnectCommand = tmp;

  m_pOnLoginEditor->getText(tmp);
  s->m_szOnLoginCommand = tmp.data();

  //crissi
  s->m_pChannelList->clear();
  int channelCount = channel_group->count();
  for (int channelNr=0;channelNr < channelCount;channelNr++)
  {
    if (((QCheckBox*)(channel_group->find(channelNr)))->isChecked())
    {
      s->m_pChannelList->append((channel_group->find(channelNr))->text());
    //  QMessageBox::information(0,"new channel",(channel_group->find(channelNr))->text());
    }

    s->setUseAutoConnect (m_pUseAutoConnect->isChecked());

  }
}

void KviServerDetailsWidget::useDefaultInitUModeToggled(bool b)
{
  m_pIMode->setEnabled(!b);
  m_pWMode->setEnabled(!b);
  m_pSMode->setEnabled(!b);
}

//void KviServerDetailsWidget::addNewChannel(QTabWidget *tw, QWidget *tab, KviIrcServer * s)
void KviServerDetailsWidget::addNewChannel()
{
  bool ok;
  QString channelName = QInputDialog::getText(
                   __tr("Add Channel - KVIrc"), __tr("Channel name:"), QLineEdit::Normal,
                   QString::null, &ok, this );
  if ( ok && !channelName.isEmpty() )
  {
    bool insert=true;
    for (int ii=0;ii< channel_group->count();ii++)
        if (((QCheckBox*)(channel_group->find(ii)))->text() == channelName)
            insert =false;
    if (insert == true) {
        server->m_pChannelList->append(channelName);
        QCheckBox *cb = new QCheckBox( channelName, this );
        cb->setChecked( true );
        channel_group->insert(cb);

        // FIXME checkbox is only shown after reopen dialog
        channel_group->adjustSize();
    }

	/* bad hack: remove old tab and put all contents in a new tab and insert it */
	/*tw->removePage ( tab );

	// after login join channels
  QWidget *tab2 = new QWidget(tw);
  QBoxLayout *layout1 = new QBoxLayout (tab2,QBoxLayout::Down);
  QScrollView *scroll_channel_join = new QScrollView(tab2);
  QVBox*big_box = new QVBox(scroll_channel_join->viewport());
  channel_group = new QButtonGroup(__tr2qs("&Recent Channels"),big_box);
  channel_group->setRadioButtonExclusive ( false );

  QBoxLayout *buttonLayout = new QBoxLayout (channel_group,QBoxLayout::Down);

  // stored channels
  for(QStringList::Iterator it = s->m_pChannelList->begin(); it != s->m_pChannelList->end(); ++it)
  {
    QCheckBox *cb = new QCheckBox(*it , channel_group );
    buttonLayout->addWidget(cb);
    cb->setChecked( true );
  }

  // recent but not stored
  for(QStringList::Iterator it2 = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin(); it2 != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end(); ++it2)
  {
    bool insert=true;
    for (int ii=0;ii< channel_group->count();ii++)
        if (((QCheckBox*)(channel_group->find(ii)))->text() == *it2)
            insert =false;
    if (insert == true) {
        QCheckBox *cb = new QCheckBox( *it2, channel_group );
        buttonLayout->addWidget(cb);
        cb->setChecked( false );
    }
  }

  QPushButton *BtnNewChannel = new QPushButton("BtnNewChannel",tab2);
  BtnNewChannel->setText(__tr("&Add Channel"));
  connect(BtnNewChannel,SIGNAL(clicked()),this, SLOT(addNewChannel(tw, tab2, s)));


  channel_group->resize(tab2->size());

  scroll_channel_join->addChild(channel_group);
  layout1->addWidget(scroll_channel_join);
  layout1->addWidget(BtnNewChannel);




  /*
      int i=0;
  	for(QStringList::Iterator it = KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).begin(); it != KVI_OPTION_STRINGLIST(KviOption_stringlistRecentChannels).end(); ++it)
  	{
          QCheckBox *tmp = new QCheckBox(*it,channel_group);
          tmp->setChecked(false);
          //QMessageBox::information(0,"neuer chan",*it);
          //channel_group->resize(channel_group->sizeHint());
  	}
  */

  /*
  #ifdef COMPILE_INFO_TIPS
  	QToolTip::add(m_pOnLoginEditor,__tr2qs("<center>The following commands will be executed after a succesfull login to this server.<br>" \
  				"This is useful for automatically joining channels, opening queries etc...</center>"));
  #endif
  */

  /*
  tw->insertTab(tab2,__tr2qs("Join Channels"),2);
  delete tab;
	*/
  }

}

// kvi_app.cpp
extern KVIRC_API KviIrcServerDataBase * g_pIrcServerDataBase;

KviServerOptionsListViewItem::KviServerOptionsListViewItem(QListView *parent,const QPixmap &pm,const char * netname)
    : QListViewItem(parent,QString(netname))
{
  setPixmap(0,pm);
  m_pServerData = 0;
}

KviServerOptionsListViewItem::KviServerOptionsListViewItem(QListViewItem *parent,const QPixmap &pm,const KviIrcServer *s)
    : QListViewItem(parent)
{
  setPixmap(0,pm);
  m_pServerData = new KviIrcServer(*s);
  setText(0,s->m_szHostname.ptr());
  setText(1,s->m_szDescription.ptr());
}

KviServerOptionsListViewItem::~KviServerOptionsListViewItem()
{
  if(m_pServerData)delete m_pServerData;
}









KviServerOptionsWidget::KviServerOptionsWidget(QWidget * parent)
    : KviOptionsWidget(parent,"server_options_widget")
{
  createLayout(7,4);

  layout()->setSpacing(4);
  layout()->setMargin(8);

  m_pContextPopup = new QPopupMenu(this);
  m_pImportPopup = new QPopupMenu(this);

  connect(m_pImportPopup,SIGNAL(aboutToShow()),this,SLOT(importPopupAboutToShow()));
  connect(m_pImportPopup,SIGNAL(activated(int)),this,SLOT(importPopupActivated(int)));


  m_pDetailsDialog = 0;
  m_pImportFilter = 0;

  m_pListView = new QListView(this);
  layout()->addMultiCellWidget(m_pListView,0,0,0,2);
  m_pListView->addColumn(__tr2qs("Server"));
  m_pListView->addColumn(__tr2qs("Description"));
  m_pListView->setRootIsDecorated(true);
  m_pListView->setAllColumnsShowFocus(true);
  m_pListView->setSelectionMode(QListView::Single);
  connect(m_pListView,SIGNAL(selectionChanged(QListViewItem *)),
          this,SLOT(listViewItemSelectionChanged(QListViewItem *)));
  connect(m_pListView,SIGNAL(rightButtonPressed(QListViewItem *,const QPoint &,int)),
          this,SLOT(listViewRightButtonPressed(QListViewItem *,const QPoint &,int)));
  connect(m_pListView,SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int )),
        this,SLOT(detailsClicked()));

#ifdef COMPILE_INFO_TIPS
  QString tiptxt = __tr2qs("<center>This is the list of available IRC servers.<br>" \
                           "Right-click on the list to add or remove servers and perform other actions.<br>"\
                           "Double-click on a item for advanced options.</center>");
  QToolTip::add(m_pListView,tiptxt);
  QToolTip::add(m_pListView->viewport(),tiptxt);
#endif

  QVBox * vbox = new QVBox(this);
  layout()->addWidget(vbox,0,3);

  m_pNewNetworkButton = new QToolButton(vbox);
  m_pNewNetworkButton->setPixmap(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_WORLD)));
  m_pNewNetworkButton->setAutoRaise(true);
  connect(m_pNewNetworkButton,SIGNAL(clicked()),this,SLOT(newNetwork()));
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pNewNetworkButton,__tr2qs("This button adds a new network."));
#endif

  m_pNewServerButton = new QToolButton(vbox);
  m_pNewServerButton->setPixmap(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVER)));
  m_pNewServerButton->setAutoRaise(true);
  connect(m_pNewServerButton,SIGNAL(clicked()),this,SLOT(newServer()));
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pNewServerButton,__tr2qs("This button adds a new server to the current network."));
#endif

  m_pRemoveButton = new QToolButton(vbox);
  m_pRemoveButton->setPixmap(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CUT)));
  m_pRemoveButton->setEnabled(false);
  m_pRemoveButton->setAutoRaise(true);
  connect(m_pRemoveButton,SIGNAL(clicked()),this,SLOT(removeCurrent()));
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pRemoveButton,__tr2qs("Removes the current network or server."));
#endif

  QFrame * f = new QFrame(vbox);
  f->setFrameStyle(QFrame::Sunken | QFrame::HLine);

  m_pCopyServerButton = new QToolButton(vbox);
  m_pCopyServerButton->setPixmap(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_COPY)));
  m_pCopyServerButton->setEnabled(false);
  m_pCopyServerButton->setAutoRaise(true);
  connect(m_pCopyServerButton,SIGNAL(clicked()),this,SLOT(copyServer()));
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pCopyServerButton,__tr2qs("Puts a copy of the current server in the clipboard."));
#endif

  m_pPasteServerButton = new QToolButton(vbox);
  m_pPasteServerButton->setPixmap(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_PASTE)));
  m_pPasteServerButton->setEnabled(false);
  m_pPasteServerButton->setAutoRaise(true);
  connect(m_pPasteServerButton,SIGNAL(clicked()),this,SLOT(pasteServer()));
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pPasteServerButton,__tr2qs("Inserts a server from the clipboard."));
#endif

  f = new QFrame(vbox);
  f->setFrameStyle(QFrame::Sunken | QFrame::HLine);

  m_pImportButton = new QToolButton(vbox);
  m_pImportButton->setPixmap(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_FOLDER)));
  m_pImportButton->setAutoRaise(true);
  m_pImportButton->setPopup(m_pImportPopup);
  m_pImportButton->setPopupDelay(1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pImportButton,__tr2qs("Allows importing server lists from external sources."));
#endif

  QFrame * lll = new QFrame(vbox);
  vbox->setStretchFactor(lll,100);



  m_pSrvNetLabel = new QLabel(__tr2qs("Server:"),this);
  layout()->addWidget(m_pSrvNetLabel,1,0);

  m_pSrvNetEdit = new QLineEdit(this);
  layout()->addWidget(m_pSrvNetEdit,1,1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pSrvNetEdit,__tr2qs("<center>This is the <b>hostname</b> of the currently selected server.</center>"));
#endif


  m_pIpV6Check = new QCheckBox(__tr2qs("Use IPv6 protocol"),this);

  layout()->addMultiCellWidget(m_pIpV6Check,1,1,2,3);
#ifndef COMPILE_IPV6_SUPPORT
  m_pIpV6Check->setEnabled(false);
#endif

#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pIpV6Check,__tr2qs("<center>This check identifies IPv6 servers.<br>If enabled, KVIrc will attempt to use the IPv6 protocol " \
                                       "(thus your OS <b>must</b> have a working IPv6 stack and you <b>must</b> have an IPv6 connection).</center>"));
#endif


  m_pPortLabel = new QLabel(__tr2qs("Port:"),this);
  layout()->addWidget(m_pPortLabel,2,0);

  m_pPortEdit = new QLineEdit(this);
  layout()->addWidget(m_pPortEdit,2,1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pPortEdit,__tr2qs("<center>This is the default <b>port</b> that this server will be contacted on.<br>Usually <b>6667</b> is OK.</center>"));
#endif

  m_pDetailsButton = new QPushButton(__tr2qs("Advanced..."),this);
  connect(m_pDetailsButton,SIGNAL(clicked()),this,SLOT(detailsClicked()));
  layout()->addMultiCellWidget(m_pDetailsButton,2,2,2,3);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pDetailsButton,__tr2qs("<center>Click here to edit advanced options for this server.</center>"));
#endif

  addSeparator(0,3,3,3);


  m_pConnectCurrent = new QPushButton(__tr2qs("Connect &Now"),this);
  layout()->addMultiCellWidget(m_pConnectCurrent,4,4,0,2);
  connect(m_pConnectCurrent,SIGNAL(clicked()),this,SLOT(connectCurrentClicked()));

#ifdef COMPILE_INFO_TIPS
  QToolTip::add(m_pConnectCurrent,__tr2qs("<center>Hit this button to connect to the currently selected server.</center>"));
#endif

  m_pRecentPopup = new QPopupMenu(this);
  connect(m_pRecentPopup,SIGNAL(aboutToShow()),this,SLOT(recentServersPopupAboutToShow()));
  connect(m_pRecentPopup,SIGNAL(activated(int)),this,SLOT(recentServersPopupClicked(int)));

  QToolButton * tb = new QToolButton(this);
  layout()->addWidget(tb,4,3);
  tb->setPixmap(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_TIME)));
  tb->setPopup(m_pRecentPopup);
  tb->setPopupDelay(1);
#ifdef COMPILE_INFO_TIPS
  QToolTip::add(tb,__tr2qs("<center>This button shows a list of recently used servers. It allows you to quickly find them in the list.</center>"));
#endif


  addSeparator(0,5,3,5);


  KviBoolSelector * b = addBoolSelector(0,6,3,6,__tr2qs("Show this dialog at startup"),KviOption_boolShowServersConnectDialogOnStart);

#ifdef COMPILE_INFO_TIPS
  QToolTip::add(b,__tr2qs("<center>If this option is enabled, the Servers dialog will appear every time you start KVIrc</center>"));
#endif

  //	KviBoolSelector * c = addBoolSelector(0,6,2,6,__tr("Close after connection"),KviOption_boolCloseServerWidgetAfterConnect);

  m_pLastEditedItem = 0;

  m_pClipboard = 0;

  fillServerList();

  layout()->setRowStretch(0,1);
  layout()->setColStretch(1,5);
  layout()->setColStretch(2,2);
  layout()->setColStretch(0,1);
  setMinimumWidth(320);
}

KviServerOptionsWidget::~KviServerOptionsWidget()
{
  if(m_pImportFilter)
  {
    disconnect(m_pImportFilter,0,this,0);
    m_pImportFilter->die();
    m_pImportFilter = 0;
  }

  if(m_pClipboard)delete m_pClipboard;
  if(m_pDetailsDialog)delete m_pDetailsDialog;
}

void KviServerOptionsWidget::recentServersPopupAboutToShow()
{
  g_pApp->fillRecentServersPopup(m_pRecentPopup);
}

void KviServerOptionsWidget::recentServersPopupClicked(int id)
{
  KviStr data = m_pRecentPopup->text(id);
  data.cutToFirst('/');
  while(data.firstCharIs('/'))data.cutLeft(1);
  data.cutFromLast(':');
  // we should have a full server name now , with no port

  for(QListViewItem * net = m_pListView->firstChild();net;net = net->nextSibling())
  {
    for(QListViewItem * srv = net->firstChild();srv;srv = srv->nextSibling())
    {
      if(kvi_strEqualCI(((KviServerOptionsListViewItem *)srv)->m_pServerData->hostname(),data.ptr()))
      {
        net->setOpen(true);
        m_pListView->setCurrentItem(srv);
        m_pListView->ensureItemVisible(srv);
        return;
      }
    }
  }
  // FIXME: add the server if it isn't there ?
}

/*
void KviServerOptionsWidget::parseCommand(const KviStr & szCommand,KviStr & szServer,KviStr & szPort)
{
    KviStr szServerAndPort = szCommand;
    szServerAndPort.cutLeft(7); // "server"
    szServerAndPort.cutRight(1); // ";"
    bool bOk;
    int port = szServerAndPort.right(5).toUShort(&bOk);
    if(bOk)
    {
        szServer = szServerAndPort.cutRight(5);
        szPort.sprintf("%d",port);
    }
}
*/
/*
void KviServerOptionsWidget::itemRecentClicked(QListBoxItem * it)
{
//	saveLastItem();
	m_pLastEditedItem = 0;

	m_pSrvNetLabel->setEnabled(false);
	m_pSrvNetEdit->setEnabled(false);
	m_pDetailsButton->setEnabled(false);
	m_pPortLabel->setEnabled(false);
	m_pPortEdit->setEnabled(false);
	m_pIpV6Check->setEnabled(false);
	m_pConnectNew->setEnabled(true);

	m_pSrvNetLabel->setText(__tr("Server:"));
	//FIXME XXX: Put the IPV6 check in the correct state


	if (it == 0)return;
	m_pListView->clearSelection();
	KviStr text = it->text();
	KviStr szServer = "";
	KviStr szPort = "";
	if(text.hasData())
	{
		KviStr szCommand;
		if(parseIrcUrl(text.ptr(),szCommand,KVI_IRCURL_CONTEXT_THIS))
		{
			parseCommand(szCommand,szServer,szPort);
			m_pSrvNetEdit->setText(szServer.ptr());
			m_pPortEdit->setText(szPort.ptr());
		}
	}
}

*/
/*
void KviServerOptionsWidget::itemRecentDoubleClicked(QListBoxItem * it)
{
    if (it == 0)return;
    m_pListView->clearSelection();
    KviStr text = it->text();
    KviStr szServer = "";
    KviStr szPort = "";
    if(text.hasData())
    {
        KviStr szCommand;
        if(parseIrcUrl(text.ptr(),szCommand,KVI_IRCURL_CONTEXT_THIS))
        {
            parseCommand(szCommand,szServer,szPort);
            m_pSrvNetEdit->setText(szServer.ptr());
            m_pPortEdit->setText(szPort.ptr());
            g_pUserParser->parseCommandBuffer(szCommand.ptr(),g_pApp->activeWindow());
        }
    }
}
*/

void KviServerOptionsWidget::connectCurrentClicked()
{
  saveLastItem();
  commit();
  KviStr szServer = m_pSrvNetEdit->text();
  if (szServer.isEmpty())return;
  g_pUserParser->parseCommandBuffer("server -u",g_pApp->activeConsole());
}

/*
void KviServerOptionsWidget::connectNewClicked()
{
  saveLastItem();
  commit();
  KviStr szServer = m_pSrvNetEdit->text();
  if (szServer.isEmpty())return;

  g_pUserParser->parseCommandBuffer("server -n",g_pApp->activeConsole());
}
*/

void KviServerOptionsWidget::fillServerList()
{
  KviServerOptionsListViewItem * net;
  KviServerOptionsListViewItem * srv;

  KviPtrList<KviIrcNetwork> * l = g_pIrcServerDataBase->networkList();

  for(KviIrcNetwork * n = l->first();n;n = l->next())
  {
    net = new KviServerOptionsListViewItem(m_pListView,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_WORLD)),n->name());
    KviPtrList<KviIrcServer> * sl = n->serverList();
    net->setOpen(n == g_pIrcServerDataBase->currentNetwork());
    //debug("Current server = %s",n->currentServer() ? n->currentServer()->m_szHostname.ptr() : "non");
    for(KviIrcServer * s = sl->first();s;s = sl->next())
    {
      srv = new KviServerOptionsListViewItem(net,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVER)),s);

      if((s == n->currentServer()) && (n == g_pIrcServerDataBase->currentNetwork()))
      {
        m_pListView->setSelected(srv,true);
        m_pListView->ensureItemVisible(srv);
      }
    }
  }
}

void KviServerOptionsWidget::listViewItemSelectionChanged(QListViewItem *it)
{
  //	m_pRecentListBox->clearSelection();
  saveLastItem();
  m_pLastEditedItem = (KviServerOptionsListViewItem *)it;

  if(m_pLastEditedItem)
  {
    m_pSrvNetLabel->setEnabled(true);
    m_pSrvNetEdit->setEnabled(true);
    m_pDetailsButton->setEnabled(m_pLastEditedItem->m_pServerData);
    m_pPortLabel->setEnabled(m_pLastEditedItem->m_pServerData);
    m_pPortEdit->setEnabled(m_pLastEditedItem->m_pServerData);
#ifdef COMPILE_IPV6_SUPPORT
    m_pIpV6Check->setEnabled(m_pLastEditedItem->m_pServerData);
#else
    m_pIpV6Check->setEnabled(false);
#endif
    m_pConnectCurrent->setEnabled(m_pLastEditedItem->m_pServerData);
    //m_pConnectNew->setEnabled(m_pLastEditedItem->m_pServerData);

    m_pRemoveButton->setEnabled(true);
    m_pCopyServerButton->setEnabled(m_pLastEditedItem->m_pServerData);

    if(m_pLastEditedItem->m_pServerData)
    {
      m_pSrvNetLabel->setText(__tr("Server:"));
      m_pSrvNetEdit->setText(m_pLastEditedItem->m_pServerData->m_szHostname.ptr());
      KviStr tmp(KviStr::Format,"%u",m_pLastEditedItem->m_pServerData->m_uPort);
      m_pPortEdit->setText(tmp.ptr());
#ifdef COMPILE_IPV6_SUPPORT
      m_pIpV6Check->setChecked(m_pLastEditedItem->m_pServerData->isIpV6());
#else
      m_pIpV6Check->setChecked(false);
#endif

    }
    else
    {
      m_pSrvNetLabel->setText(__c2q(__tr("Network:")));
      m_pSrvNetEdit->setText(it->text(0));
      m_pPortEdit->setText("");
    }
  }
  else
  {
    m_pRemoveButton->setEnabled(false);
    m_pCopyServerButton->setEnabled(true);

    m_pConnectCurrent->setEnabled(false);
    //m_pConnectNew->setEnabled(false);
    m_pIpV6Check->setEnabled(false);
    m_pSrvNetLabel->setEnabled(false);
    m_pSrvNetEdit->setEnabled(false);
    m_pSrvNetEdit->setText(__tr("No selection"));
    m_pDetailsButton->setEnabled(false);
    m_pPortLabel->setEnabled(false);
    m_pPortEdit->setEnabled(false);
    m_pPortEdit->setText("");
  }
}



void KviServerOptionsWidget::saveLastItem()
{
  if(m_pLastEditedItem)
  {
    if(m_pLastEditedItem->m_pServerData)
    {
      KviStr tmp = m_pSrvNetEdit->text();
      if(tmp.isEmpty())tmp = "irc.unknown.net";
      m_pLastEditedItem->setText(0,tmp.ptr());
      m_pLastEditedItem->m_pServerData->m_szHostname = tmp;
      m_pLastEditedItem->setText(1,m_pLastEditedItem->m_pServerData->m_szDescription.ptr());
      tmp = m_pPortEdit->text();
      bool bOk;
      unsigned short int uPort = tmp.toUShort(&bOk);
      if(!bOk)uPort = 6667;
      m_pLastEditedItem->m_pServerData->m_uPort = uPort;
#ifdef COMPILE_IPV6_SUPPORT
      m_pLastEditedItem->m_pServerData->setIpV6(m_pIpV6Check->isChecked());
#else
      m_pLastEditedItem->m_pServerData->setIpV6(false);
#endif

    }
    else
    {
      KviStr tmp = m_pSrvNetEdit->text();
      if(tmp.isEmpty())tmp = __tr("UnknownNet");
      m_pLastEditedItem->setText(0,tmp.ptr());
    }
  }
}

void KviServerOptionsWidget::commit()
{
  //debug("COMMITTING");
  saveLastItem();
  g_pIrcServerDataBase->clear();
  KviServerOptionsListViewItem * it = (KviServerOptionsListViewItem *)m_pListView->firstChild();
  while(it)
  {
    KviStr tmp = it->text(0);
    if(tmp.hasData())
    {
      KviIrcNetwork * net = g_pIrcServerDataBase->findNetwork(tmp.ptr());
      if(!net)
      {
        net = new KviIrcNetwork(tmp.ptr());
        g_pIrcServerDataBase->insertNetwork(net);
      }
      if(it == m_pLastEditedItem)g_pIrcServerDataBase->setCurrentNetwork(net);

      KviServerOptionsListViewItem * ch = (KviServerOptionsListViewItem *)it->firstChild();
      KviIrcServer *srv;
      while(ch)
      {
        if(ch->m_pServerData)
        {
          if(ch->m_pServerData->m_szHostname.hasData())
          {
            srv = net->findServer(ch->m_pServerData->m_szHostname.ptr(),ch->m_pServerData->m_uPort);
            if(!srv)
            {
              srv = new KviIrcServer(*(ch->m_pServerData));
              net->insertServer(srv);
            }
            else *srv = *(ch->m_pServerData);
            if(ch == m_pLastEditedItem)
            {
              g_pIrcServerDataBase->setCurrentNetwork(net);
              net->setCurrentServer(srv);
            }
          }
        }
        ch = (KviServerOptionsListViewItem *)ch->nextSibling();
      }

      if(net->currentServer() == 0)net->setCurrentServer(net->serverList()->first());
    }
    it = (KviServerOptionsListViewItem *)it->nextSibling();
  }
  if(g_pIrcServerDataBase->currentNetwork() == 0)
  {
    g_pIrcServerDataBase->setCurrentNetwork(g_pIrcServerDataBase->networkList()->first());
  }

  KviOptionsWidget::commit();
}

void KviServerOptionsWidget::listViewRightButtonPressed(QListViewItem *it,const QPoint &pnt,int col)
{
  m_pContextPopup->clear();
  m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_WORLD)),__tr2qs("&New Network"),this,SLOT(newNetwork()));
  if(it)
  {
    if(((KviServerOptionsListViewItem *)it)->m_pServerData)
    {
      m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CUT)),__tr2qs("Re&move Server"),this,SLOT(removeCurrent()));
      m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_COPY)),__tr2qs("&Copy Server"),this,SLOT(copyServer()));
    }
    else
    {
      m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_CUT)),__tr2qs("Re&move Network"),this,SLOT(removeCurrent()));
    }
    m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVER)),__tr2qs("New &Server"),this,SLOT(newServer()));
    if(m_pClipboard)m_pContextPopup->insertItem(*(g_pIconManager->getSmallIcon(KVI_SMALLICON_PASTE)),__tr2qs("Paste server"),this,SLOT(pasteServer()));
  }
  m_pContextPopup->insertSeparator();
  //	m_pContextPopup->insertItem(__c2q(__tr("Merge list from server.ini")),this,SLOT(importFromIni()));
  m_pContextPopup->insertItem(__tr2qs("Clear List"),this,SLOT(clearList()));
  m_pContextPopup->insertSeparator();
  m_pContextPopup->insertItem(__tr2qs("&Import List..."),m_pImportPopup);
  m_pContextPopup->popup(QCursor::pos());
}

void KviServerOptionsWidget::importPopupAboutToShow()
{
  g_pModuleManager->loadModulesByCaps("serverimport");
  m_pImportPopup->clear();

  KviModuleExtensionDescriptorList * l = KviModuleExtensionManager::instance()->getExtensionList("serverimport");

  if(!l)return;

  int id;

  for(KviModuleExtensionDescriptor * d = l->first();d;d = l->next())
  {
    if(d->icon())
      id = m_pImportPopup->insertItem(*(d->icon()),d->visibleName().ptr());
    else
      id = m_pImportPopup->insertItem(d->visibleName().ptr());
    m_pImportPopup->setItemParameter(id,d->id());
  }

  /*
  	if(!g_pServerImportFilters)return;

  	// FIXME: Ensure that the filters are REAL filters: are able to provide an import filter object
  	QAsciiDictIterator<KviStr> it(*g_pServerImportFilters);
  	while(it.current())
  	{
  		m_pImportPopup->insertItem(it.currentKey());
  		++it;
  	}
  */
}

void KviServerOptionsWidget::importPopupActivated(int id)
{
  // ensure that we have all the modules : they could have been unloaded while the popup was displayed
  g_pModuleManager->loadModulesByCaps("serverimport");

  KviModuleExtensionDescriptorList * l = KviModuleExtensionManager::instance()->getExtensionList("serverimport");

  if(!l)
  {
    // ops.. internal error: I thought to have a module capable of importing servers
    // but actually it's not the case.. something weird happened (in the best case
    // the user has just unloaded the module and removed it from disk ?)
    // FIXME: KviMessageBox::warning
    kvi_warningBox(__tr("Ops.. something weird happened:<br>Can't find any module capable of importing servers"));
    return;
  }

  if(m_pImportFilter)
  {
    disconnect(m_pImportFilter,0,this,0);
    m_pImportFilter->die();
    m_pImportFilter = 0;
  }

  id = m_pImportPopup->itemParameter(id);

  m_pImportFilter = (KviMexServerImport *)KviModuleExtensionManager::instance()->allocateExtension("serverimport",id,0);

  if(!m_pImportFilter)
  {
    kvi_warningBox(__tr("Ops... something weird happened:<br>Can't find the module that was capable of this import action :("));
    return;
  }

  connect(m_pImportFilter,SIGNAL(server(const KviIrcServer &,const char *)),this,SLOT(importServer(const KviIrcServer &,const char *)));
  connect(m_pImportFilter,SIGNAL(destroyed()),this,SLOT(importerDead()));

  m_pImportFilter->start();
}

void KviServerOptionsWidget::importerDead()
{
  m_pImportFilter = 0;
}

void KviServerOptionsWidget::importServer(const KviIrcServer &s,const char * network)
{
  KviServerOptionsListViewItem * net = findNetItem(network);
  if(!net)
  {
    net = new KviServerOptionsListViewItem(m_pListView,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_WORLD)),network);
    net->setOpen(true);
  }

  for(KviServerOptionsListViewItem * srv = (KviServerOptionsListViewItem *)net->firstChild();srv;srv = (KviServerOptionsListViewItem *)srv->nextSibling())
  {
    if((srv)->m_pServerData->useSSL() == s.useSSL())
    {
      if(srv->m_pServerData->isIpV6() == s.isIpV6())
      {
        if(kvi_strEqualCI(srv->m_pServerData->hostname(),s.hostname()))
        {
          // update the port
          srv->m_pServerData->setPort(s.port());
          if(s.ipAddress().hasData())srv->m_pServerData->setIpAddress(s.ipAddress());
          if(s.password().hasData())srv->m_pServerData->setPassword(s.password());
          if(s.nickname().hasData())srv->m_pServerData->setNickname(s.nickname());
          m_pListView->setCurrentItem(srv);
          m_pListView->ensureItemVisible(srv);
          return;
        }
      }
    }
  }

  // not found : add it!
  KviServerOptionsListViewItem * newServer = new KviServerOptionsListViewItem(net,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVER)),&s);
  m_pListView->setCurrentItem(newServer);
  m_pListView->ensureItemVisible(newServer);

}

void KviServerOptionsWidget::newNetwork()
{
  KviServerOptionsListViewItem * it = new KviServerOptionsListViewItem(m_pListView,*(g_pIconManager->getSmallIcon(KVI_SMALLICON_WORLD)),__tr("New Network"));
  it->setOpen(true);
  m_pListView->setSelected(it,true);
  m_pListView->ensureItemVisible(it);
}

void KviServerOptionsWidget::newServer()
{
  if(m_pLastEditedItem)
  {
    KviServerOptionsListViewItem * net;
    if(m_pLastEditedItem->m_pServerData)
    {
      net = (KviServerOptionsListViewItem *)m_pLastEditedItem->parent();
      if(!net)return;
    }
    else net = m_pLastEditedItem;

    KviIrcServer tmp;
    tmp.m_szHostname = __tr("irc.unknown.net");
    tmp.setCacheIp(true);

    KviServerOptionsListViewItem * it = new KviServerOptionsListViewItem(net,
                                        *(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVER)),&tmp);

    net->setOpen(true);

    m_pListView->setSelected(it,true);
    m_pListView->ensureItemVisible(it);
  }
}

void KviServerOptionsWidget::copyServer()
{
  if(m_pLastEditedItem)
  {
    if(m_pLastEditedItem->m_pServerData)
    {
      if(!m_pClipboard)m_pClipboard = new KviIrcServer();
      *m_pClipboard = *(m_pLastEditedItem->m_pServerData);
      m_pPasteServerButton->setEnabled(true);
    }
  }
}

void KviServerOptionsWidget::pasteServer()
{
  if(m_pClipboard)
  {
    if(m_pLastEditedItem)
    {
      KviServerOptionsListViewItem * net;
      if(m_pLastEditedItem->m_pServerData)
      {
        net = (KviServerOptionsListViewItem *)m_pLastEditedItem->parent();
        if(!net)return;
      }
      else net = m_pLastEditedItem;

      KviServerOptionsListViewItem * it = new KviServerOptionsListViewItem(net,
                                          *(g_pIconManager->getSmallIcon(KVI_SMALLICON_SERVER)),m_pClipboard);

      net->setOpen(true);

      m_pListView->setSelected(it,true);
      m_pListView->ensureItemVisible(it);
    }
  }
}

void KviServerOptionsWidget::removeCurrent()
{
  if(m_pLastEditedItem)
  {
    delete m_pLastEditedItem;
    m_pLastEditedItem = 0;
    QListViewItem * it = m_pListView->firstChild();
    if(it)
    {
      m_pListView->setSelected(it,true);
      m_pListView->ensureItemVisible(it);
    }
  }
}



KviServerOptionsListViewItem * KviServerOptionsWidget::findNetItem(const char * netname)
{
  KviServerOptionsListViewItem * it = (KviServerOptionsListViewItem *)m_pListView->firstChild();
  while(it)
  {
    KviStr szNetName = it->text(0);
    if(kvi_strEqualCI(szNetName.ptr(),netname))return it;
    it = (KviServerOptionsListViewItem *)it->nextSibling();
  }
  return 0;
}

void KviServerOptionsWidget::clearList()
{
  m_pListView->clear();
  m_pLastEditedItem = 0;
  listViewItemSelectionChanged(0);
}

void KviServerOptionsWidget::detailsClicked()
{
  if(!m_pLastEditedItem)return;
  if(!(m_pLastEditedItem->m_pServerData))return;

  saveLastItem();

  m_pDetailsDialog = new KviServerDetailsWidget(this,m_pLastEditedItem->m_pServerData);

  int retCode = m_pDetailsDialog->exec();
  if(retCode == QDialog::Accepted)
  {
    if(m_pLastEditedItem)
    {
      if(m_pLastEditedItem->m_pServerData)
      {
        m_pDetailsDialog->fillData(m_pLastEditedItem->m_pServerData);
        saveLastItem();
      }
    }
  }


  delete m_pDetailsDialog;
  m_pDetailsDialog = 0;
}

#include "m_optw_servers.moc"
