//
//   File : optw_transport.cpp
//   Creation date : Mon Nov 13 2000 14:14:22 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "optw_transport.h"

#include <qgroupbox.h>

#include "kvi_options.h"
#include "kvi_locale.h"

KviTransportOptionsWidget::KviTransportOptionsWidget(QWidget * parent)
: KviOptionsWidget(parent,"transport_options_widget")
{
	createLayout(4,2);
	QGroupBox * gbox = new QGroupBox(1,QGroupBox::Horizontal,__tr("Timeout Values" ),this);
	KviUIntSelector * u = new KviUIntSelector(gbox,__tr2qs("Server connect:"),
		&(KVI_OPTION_UINT(KviOption_uintIrcSocketTimeout)),5,6000,60,true);
	u->setSuffix(__tr(" sec"));
	u = new KviUIntSelector(gbox,__tr2qs("Outgoing data queue flush:"),
		&(KVI_OPTION_UINT(KviOption_uintSocketQueueFlushTimeout)),100,2000,500,true);
	u->setSuffix(__tr(" msec"));
	addWidgetToLayout(gbox,0,0,0,0);
	gbox = new QGroupBox(1,QGroupBox::Horizontal,__tr("Limits" ),this);
	KviBoolSelector * b = new KviBoolSelector(gbox,__tr2qs("Limit outgoing"),
		&(KVI_OPTION_BOOL(KviOption_boolLimitOutgoingTraffic)),true);
	u = new KviUIntSelector(gbox,__tr2qs("1 message per:"),
		&(KVI_OPTION_UINT(KviOption_uintOutgoingTrafficLimitUSeconds)),10000,2000000,10000000,
		KVI_OPTION_BOOL(KviOption_boolLimitOutgoingTraffic));
	u->setSuffix(__tr(" usec"));
	connect(b,SIGNAL(toggled(bool)),u,SLOT(setEnabled(bool)));
	addWidgetToLayout(gbox,0,1,0,1);
	gbox = new QGroupBox(2,QGroupBox::Horizontal,__tr("Interfaces" ),this);
	b = new KviBoolSelector(gbox,__tr2qs("Bind IPv4 connections to:"),
		&(KVI_OPTION_BOOL(KviOption_boolBindIrcIpV4ConnectionsToSpecifiedAddress)),true);
	KviStringSelector * s = new KviStringSelector(gbox,"",
		&(KVI_OPTION_STRING(KviOption_stringIpV4ConnectionBindAddress)),KVI_OPTION_BOOL(KviOption_boolBindIrcIpV4ConnectionsToSpecifiedAddress));
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
#ifdef COMPILE_IPV6_SUPPORT
	b = new KviBoolSelector(gbox,__tr2qs("Bind IPv6 connections to:"),
		&(KVI_OPTION_BOOL(KviOption_boolBindIrcIpV6ConnectionsToSpecifiedAddress)),true);
	s = new KviStringSelector(gbox,"",
		&(KVI_OPTION_STRING(KviOption_stringIpV6ConnectionBindAddress)),KVI_OPTION_BOOL(KviOption_boolBindIrcIpV6ConnectionsToSpecifiedAddress));
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	addWidgetToLayout(gbox,0,2,0,2);
	addRowSpacer(0,3,0,3);
#endif //!COMPILE_IPV6_SUPPORT
}

KviTransportOptionsWidget::~KviTransportOptionsWidget()
{
}


#include "m_optw_transport.moc"
